@@ -5,6 +5,7 @@
 */
 #include "windowsystem.h"
 #include "logging.h"
+#include "surfacehelper.h"
 #include "waylandintegration.h"
 #include "waylandxdgactivationv1_p.h"
 
@@ -12,20 +13,38 @@
 
 #include <KWayland/Client/connection_thread.h>
 #include <KWayland/Client/plasmashell.h>
-#include <KWayland/Client/plasmawindowmanagement.h>
 #include <KWayland/Client/registry.h>
 #include <KWayland/Client/seat.h>
 #include <KWayland/Client/surface.h>
 
+#include "qwayland-plasma-window-management.h"
 #include <QGuiApplication>
 #include <QPixmap>
 #include <QPoint>
 #include <QString>
+#include <QWaylandClientExtensionTemplate>
 #include <QWindow>
 #include <private/qwaylanddisplay_p.h>
 #include <private/qwaylandinputdevice_p.h>
 #include <private/qwaylandwindow_p.h>
 #include <qpa/qplatformnativeinterface.h>
+#include <qwaylandclientextension.h>
+
+class WindowManagement : public QWaylandClientExtensionTemplate<WindowManagement>, public QtWayland::org_kde_plasma_window_management
+{
+public:
+    WindowManagement()
+        : QWaylandClientExtensionTemplate<WindowManagement>(16)
+    {
+    }
+
+    void org_kde_plasma_window_management_show_desktop_changed(uint32_t state) override
+    {
+        showingDesktop = state == show_desktop_enabled;
+    }
+
+    bool showingDesktop = false;
+};
 
 using namespace KWayland::Client;
 
@@ -34,20 +53,26 @@ WindowSystem::WindowSystem()
     , KWindowSystemPrivateV2()
     , m_lastToken(qEnvironmentVariable("XDG_ACTIVATION_TOKEN"))
 {
+    m_windowManagement = new WindowManagement;
+}
+
+WindowSystem::~WindowSystem()
+{
+    delete m_windowManagement;
 }
 
 void WindowSystem::activateWindow(WId win, long int time)
 {
     Q_UNUSED(time);
-    Surface *s = Surface::fromQtWinId(win);
+    auto s = surfaceForWindow(QWindow::fromWinId(win));
     if (!s) {
         return;
     }
     WaylandXdgActivationV1 *activation = WaylandIntegration::self()->activation();
-    if (!activation) {
+    if (!activation->isActive()) {
         return;
     }
-    activation->activate(m_lastToken, *s);
+    activation->activate(m_lastToken, s);
 }
 
 void WindowSystem::forceActiveWindow(WId win, long int time)
@@ -71,7 +96,7 @@ void WindowSystem::requestToken(QWindow *window, uint32_t serial, const QString
     }(window);
 
     WaylandXdgActivationV1 *activation = WaylandIntegration::self()->activation();
-    if (!activation) {
+    if (!activation->isActive()) {
         // Ensure that xdgActivationTokenArrived is always emitted asynchronously
         QTimer::singleShot(0, [serial] {
             Q_EMIT KWindowSystem::self()->xdgActivationTokenArrived(serial, {});
@@ -145,14 +170,12 @@ int WindowSystem::currentDesktop()
     return 0;
 }
 
-#if KWINDOWSYSTEM_BUILD_DEPRECATED_SINCE(5, 101)
 void WindowSystem::demandAttention(WId win, bool set)
 {
     Q_UNUSED(win)
     Q_UNUSED(set)
     qCDebug(KWAYLAND_KWS) << "This plugin does not support demanding attention";
 }
-#endif
 
 QString WindowSystem::desktopName(int desktop)
 {
@@ -169,14 +192,12 @@ QPoint WindowSystem::desktopToViewport(int desktop, bool absolute)
     return QPoint();
 }
 
-#if KWINDOWSYSTEM_BUILD_DEPRECATED_SINCE(5, 0)
 WId WindowSystem::groupLeader(WId window)
 {
     Q_UNUSED(window)
     qCDebug(KWAYLAND_KWS) << "This plugin does not support group leader";
     return 0;
 }
-#endif
 
 bool WindowSystem::icccmCompliantMappingState()
 {
@@ -298,7 +319,6 @@ void WindowSystem::setStrut(WId win, int left, int right, int top, int bottom)
     qCDebug(KWAYLAND_KWS) << "This plugin does not support window struts";
 }
 
-#if KWINDOWSYSTEM_BUILD_DEPRECATED_SINCE(5, 101)
 void WindowSystem::setIcons(WId win, const QPixmap &icon, const QPixmap &miniIcon)
 {
     Q_UNUSED(win)
@@ -306,7 +326,6 @@ void WindowSystem::setIcons(WId win, const QPixmap &icon, const QPixmap &miniIco
     Q_UNUSED(miniIcon)
     qCDebug(KWAYLAND_KWS) << "This plugin does not support setting window icons";
 }
-#endif
 
 void WindowSystem::setOnActivities(WId win, const QStringList &activities)
 {
@@ -331,10 +350,10 @@ void WindowSystem::setOnDesktop(WId win, int desktop)
 
 void WindowSystem::setShowingDesktop(bool showing)
 {
-    if (!WaylandIntegration::self()->plasmaWindowManagement()) {
+    if (!m_windowManagement->isActive()) {
         return;
     }
-    WaylandIntegration::self()->plasmaWindowManagement()->setShowingDesktop(showing);
+    m_windowManagement->show_desktop(showing ? WindowManagement::show_desktop_enabled : WindowManagement::show_desktop_disabled);
 }
 
 void WindowSystem::clearState(WId win, NET::States state)
@@ -518,10 +537,10 @@ void WindowSystem::setUserTime(WId win, long int time)
 
 bool WindowSystem::showingDesktop()
 {
-    if (!WaylandIntegration::self()->plasmaWindowManagement()) {
+    if (!m_windowManagement->isActive()) {
         return false;
     }
-    return WaylandIntegration::self()->plasmaWindowManagement()->isShowingDesktop();
+    return m_windowManagement->showingDesktop;
 }
 
 QList<WId> WindowSystem::stackingOrder()
@@ -530,14 +549,12 @@ QList<WId> WindowSystem::stackingOrder()
     return QList<WId>();
 }
 
-#if KWINDOWSYSTEM_BUILD_DEPRECATED_SINCE(5, 0)
 WId WindowSystem::transientFor(WId window)
 {
     Q_UNUSED(window)
     qCDebug(KWAYLAND_KWS) << "This plugin does not support transient for windows";
     return 0;
 }
-#endif
 
 int WindowSystem::viewportToDesktop(const QPoint &pos)
 {
